/*
 * Copyright (C) 2010-2012 Agrosense info@agrosense.eu
 * 
 * Licensed under the Eclipse Public License - v 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *          http://www.eclipse.org/legal/epl-v10.html
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package nl.cloudfarming.client.imp;

import java.io.IOException;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.Action;
import org.netbeans.api.project.Project;
import org.netbeans.spi.project.ui.support.NodeFactory;
import org.netbeans.spi.project.ui.support.NodeFactorySupport;
import org.netbeans.spi.project.ui.support.NodeList;
import org.openide.filesystems.FileObject;
import org.openide.loaders.DataFolder;
import org.openide.nodes.ChildFactory;
import org.openide.nodes.Node;
import org.openide.util.*;
import org.openide.util.Lookup.Result;

/**
 *
 * @author Timon Veenstra <monezz@gmail.com>
 */
@NodeFactory.Registration(projectType = "nl-cloudfarming-client-farm")
@NbBundle.Messages({
    "ImportNode.name=Import"
})
public class ImportNodeFactory implements NodeFactory {

    private static final Logger LOGGER = Logger.getLogger("nl.cloudfarming.client.field.fieldimport");

    @Override
    public NodeList<?> createNodes(Project project) {
        // add import root node to project
        FileObject importFolder;
        try {
            importFolder = ImportProcessor.Util.getImportFolder(project);
            DataFolder importDataObject = DataFolder.findFolder(importFolder);
            return NodeFactorySupport.fixedNodeList(new ImportRootNode(importDataObject.getNodeDelegate(), project));
        } catch (IOException ex) {
            LOGGER.log(Level.SEVERE, "Error while creating root import node {0}", ex.getMessage());
            Exceptions.printStackTrace(ex);
            return NodeFactorySupport.fixedNodeList();
        }
    }

    private static final class ImportRootNode extends AbstractImportFolderNode {

        private Project project;
        
        public ImportRootNode(Node original, Project project) {
//            super(original, NodeFactorySupport.createCompositeChildren(project, "Projects/nl-cloudfarming-client-import/Nodes"));
            super(project, original, Children.create(new ImportNodeChildFactory(project), true));
            this.project = project;
            setName(Bundle.ImportNode_name());
            setDisplayName(Bundle.ImportNode_name());
        }

        @Override
        public Action[] getActions(boolean context) {
            return new Action[]{new ImportAction(project)};
        }
    }

    private static final class ImportNodeChildFactory extends ChildFactory<Node> {

        private final Project project;
        private static final Result<ImportProcessor> processorResult = Lookup.getDefault().lookupResult(ImportProcessor.class);
        
        public ImportNodeChildFactory(Project project) {
            this.project = project;
            processorResult.addLookupListener(new LookupListener() {
                @Override
                public void resultChanged(LookupEvent ev) {
                    refresh(true);
                }
            });
        }

        @Override
        protected Node createNodeForKey(Node key) {
            return key;
        }

        @Override
        protected boolean createKeys(List<Node> toPopulate) {
            Set<Node> nodes = new HashSet<>();
            for (ImportProcessor importProcessor: processorResult.allInstances()){
                nodes.add(importProcessor.getImportNode(project));
            }
            toPopulate.addAll(nodes);
            return true;
        }
    }
}
